@@ -2,6 +2,7 @@
 DATABRICKS_HOST, DATABRICKS_TOKEN
 """
 import click
+import pkg_resources
 from rich import print
 
 from .prophecy_build_tool import ProphecyBuildTool
@@ -22,6 +23,16 @@ def build(path):
     pbt = ProphecyBuildTool(path)
     pbt.build(dict())
 
+@cli.command()
+@click.option(
+    "--path",
+    help="Path to the directory containing the pbt_project.yml file",
+    required=True,
+)
+def validate(path):
+    pbt = ProphecyBuildTool(path)
+    pbt.validate()
+
 
 @cli.command()
 @click.option(
@@ -45,11 +56,22 @@ def build(path):
     help="Prophecy URL placeholder to be used during deployments",
     default="",
 )
-def deploy(path, dependent_projects_path, release_version, project_id, prophecy_url):
+@click.option(
+    "--fabric-ids",
+    help="Fabric IDs(comma separated) which can be used to filter jobs for deployments",
+    default="",
+)
+@click.option(
+    "--skip-builds",
+    is_flag=True,
+    default=False,
+    help="Flag to skip building Pipelines"
+)
+def deploy(path, dependent_projects_path, release_version, project_id, prophecy_url, fabric_ids, skip_builds):
     pbt = ProphecyBuildTool(
         path, dependent_projects_path, release_version, project_id, prophecy_url
     )
-    pbt.deploy()
+    pbt.deploy(fabric_ids, skip_builds)
 
 
 @cli.command()
@@ -65,6 +87,7 @@ def test(path):
 
 if __name__ == "pbt":
     print(
-        "[bold purple]Prophecy-build-tool[/bold purple] [bold black]v1.0.3[/bold black]\n"
+        f"[bold purple]Prophecy-build-tool[/bold purple] [bold black]"
+        f"v{pkg_resources.require('prophecy-build-tool')[0].version}[/bold black]\n"
     )
     cli()
