@@ -6,7 +6,7 @@ import subprocess
 import sys
 from glob import glob
 from os import listdir
-from os.path import normpath, basename, isfile, join, dirname
+from os.path import basename, isfile, join, dirname
 from typing import Dict
 
 import yaml
@@ -28,6 +28,7 @@ class ProphecyBuildTool:
         project_id: str = "",
         prophecy_url: str = "",
     ):
+
         if not path_root:
             self._error("Path of project not passed as argument using --path.")
         self.operating_system = sys.platform
@@ -35,24 +36,21 @@ class ProphecyBuildTool:
         self.path_project = os.path.join(self.path_root, "pbt_project.yml")
 
         self._verify_project()
-        self._verify_databricks_configs()
         self._parse_project()
         self.dependent_projects = {}
         if dependent_projects_path:
             print("\nParsing dependent projects")
             other_pbt_projects = glob(
-                f"{dependent_projects_path}**{os.sep}pbt_project.yml", recursive=True
+                f"{dependent_projects_path}{os.sep}**{os.sep}pbt_project.yml",
+                recursive=True,
             )
             for dependent_project in other_pbt_projects:
                 self.dependent_projects[dirname(dependent_project)] = ProphecyBuildTool(
                     dirname(dependent_project)
                 )
-        config = EnvironmentVariableConfigProvider().get_config()
-
-        self.api_client = _get_api_client(config)
-
-        self.dbfs_service = DbfsService(self.api_client)
-        self.jobs_service = JobsService(self.api_client)
+        self.dbfs_service = None
+        self.jobs_service = None
+        self.api_client = None
         self.python_cmd, self.pip_cmd = self.get_python_commands(path_root)
         self.pipelines_build_path = {}
         self.dependent_pipelines_build_path = {}
@@ -110,6 +108,44 @@ class ProphecyBuildTool:
             print("ERROR: python not found")
             sys.exit(1)
 
+    def validate(self):
+        pipelines = self.pipelines
+        print("\n[bold blue]Validating %s pipelines [/bold blue]" % len(pipelines))
+        overall_validate_status = True
+        for pipeline_i, (path_pipeline, pipeline) in enumerate(pipelines.items()):
+            print(
+                "\n  Validating pipeline %s [%s/%s]"
+                % (path_pipeline, pipeline_i + 1, len(pipelines))
+            )
+
+            workflow_json_path_pipeline_absolute = os.path.join(
+                os.path.join(self.path_root, path_pipeline),
+                "code",
+                ".prophecy",
+                "workflow.latest.json",
+            )
+            if os.path.exists(workflow_json_path_pipeline_absolute):
+                workflow = json.load(open(workflow_json_path_pipeline_absolute, "r"))
+                if "diagnostics" in workflow:
+                    print(
+                        f"\n[bold red]Pipeline is Broken: {pipeline['name']}[/bold red]"
+                    )
+                    overall_validate_status = False
+                else:
+                    print(
+                        f"\n[bold blue] Pipeline is validated: {pipeline['name']}[/bold blue]"
+                    )
+            else:
+                print(
+                    f"\n[bold red] Empty Pipeline Found: {pipeline['name']}![/bold red]"
+                )
+                overall_validate_status = False
+
+        if not overall_validate_status:
+            sys.exit(1)
+        else:
+            sys.exit(0)
+
     def build(self, pipelines, exit_on_build_failure=True):
         if not pipelines:
             pipelines = self.pipelines
@@ -179,18 +215,40 @@ class ProphecyBuildTool:
         else:
             return overall_build_status, self.pipelines_build_path
 
-    def deploy(self):
-        self.build(dict())
+    def deploy(self, fabric_ids: str = "", skip_builds: bool = False):
+        fabric_ids = (
+            list(i.strip() for i in fabric_ids.split(r",")) if fabric_ids else list()
+        )
+
+        self._verify_databricks_configs()
+        config = EnvironmentVariableConfigProvider().get_config()
+
+        self.api_client = _get_api_client(config)
+
+        self.dbfs_service = DbfsService(self.api_client)
+        self.jobs_service = JobsService(self.api_client)
+
+        if not skip_builds:
+            self.build(dict())
+        else:
+            print(
+                "[SKIP]: Skipping builds for all pipelines as '--skip-builds' flag is passed."
+            )
 
         print("\n[bold blue] Deploying %s jobs [/bold blue]" % self.jobs_count)
 
         pipelines_upload_failures = collections.defaultdict(list)
         job_update_failures = dict()
 
+        if not fabric_ids:
+            print("Deploying jobs for all Fabrics")
+        else:
+            print("Deploying jobs only for given Fabric IDs: %s" % (str(fabric_ids)))
+
         for job_idx, (path_job, job) in enumerate(self.jobs.items()):
             pipelines_upload_failures_job = collections.defaultdict(list)
             print(
-                "\n  Deploying job %s [%s/%s]"
+                "\n[START]:  Deploying job %s [%s/%s]"
                 % (path_job, job_idx + 1, self.jobs_count)
             )
 
@@ -215,6 +273,21 @@ class ProphecyBuildTool:
                 job_definition = json.load(_in)
 
             components = job_definition["components"]
+            fabric_id = job_definition["fabric_id"]
+
+            # if --fabric is passed, then only deploy jobs
+            if fabric_ids:
+                if (
+                    fabric_id not in fabric_ids
+                ):  # Jobs for this fabric id should be skipped
+                    print(
+                        "[SKIP]: Job skipped as it belongs to fabric id (not passed): %s"
+                        % fabric_id
+                    )
+                    continue
+                else:
+                    print("[DEPLOY]: Job being deployed for fabric id: %s" % fabric_id)
+
             generate_pipeline_config_from_pipeline_component = False
 
             for component in components:
@@ -237,7 +310,7 @@ class ProphecyBuildTool:
                             "|^.*projectSubscriptionProjectId=([0-9]+).*path=([-_.A-Za-z0-9 \/]+).*$"
                             "|^.*path=([-_.A-Za-z0-9 \/]+).*projectSubscriptionProjectId=([0-9]+).*$"
                         )
-
+                        print(f"Parsing basepipeline: {pipeline_uri}")
                         search_regex_id = re.search(
                             dependent_pipeline_regex_pattern, pipeline_uri
                         )
@@ -263,6 +336,10 @@ class ProphecyBuildTool:
                                 shared_pipeline_id
                             ] = component["PipelineComponent"]["configPath"]
 
+                            print(
+                                f"Shared pipeline id for {pipeline_uri} is {shared_pipeline_id}"
+                            )
+
                         print(
                             f"    Pipeline {pipeline_id} might be shared, checking if it exists in DBFS"
                         )
@@ -369,6 +446,7 @@ class ProphecyBuildTool:
             job_request["version"] = "2.1"
             # Upload configuration json files to dbfs
 
+            print(f"Pipeline to DBFS config map: {self.pipeline_to_dbfs_config_path}")
             local_path_dbfs_map = dict()
             if generate_pipeline_config_from_pipeline_component:
                 for (
@@ -462,7 +540,7 @@ class ProphecyBuildTool:
                 job_update_failures[job_request["name"]] = str(e)
 
         if len(pipelines_upload_failures) == 0 and len(job_update_failures) == 0:
-            print("\n[bold blue] Deployment completed successfully![/bold blue]")
+            print("\n[bold blue][DONE]: Deployment completed successfully![/bold blue]")
         else:
             print("\n[bold red] Deployment failed![/bold red]")
             if len(pipelines_upload_failures) > 0:
@@ -524,6 +602,7 @@ class ProphecyBuildTool:
             sys.exit(1)
 
     def get_python_dependencies(self, path_pipeline_absolute, path_pipeline):
+        python_dependencies = []
         if (
             Process.process_sequential(
                 [
@@ -538,28 +617,47 @@ class ProphecyBuildTool:
             )
             == 0
         ):
-            python_dependencies = []
-            egg_info_dir = f"{basename(normpath(path_pipeline))}.egg-info"
-            egg_info_requires = os.path.join(
-                *[path_pipeline_absolute, egg_info_dir, "requires.txt"]
+            egg_info_requires_glob = glob(
+                f"{path_pipeline_absolute}/**/*.egg-info/requires.txt", recursive=True
             )
-            for line in open(egg_info_requires, "r"):
-                if not line.lstrip().startswith("[") and line.strip():
-                    python_dependencies.append(line.strip())
-            return python_dependencies
+            if not egg_info_requires_glob:
+                print(f"Failed to get python dependencies for {path_pipeline}")
+            else:
+                egg_info_requires = egg_info_requires_glob[0]
+                for line in open(egg_info_requires, "r"):
+                    if not line.lstrip().startswith("[") and line.strip():
+                        python_dependencies.append(line.strip())
         else:
-            print("Failed to get python dependencies")
-            sys.exit(1)
+            print(f"Failed to get python dependencies for {path_pipeline}")
+
+        print(f"    Dependencies found for {path_pipeline}: {python_dependencies}")
+        return python_dependencies
 
     def build_python(self, path_pipeline_absolute, path_pipeline):
         python_dependencies = self.get_python_dependencies(
             path_pipeline_absolute, path_pipeline
         )
+
+        install_python_dependencies_cmd = (
+            [self.pip_cmd, "install", "-q"] + python_dependencies
+            if python_dependencies
+            else [
+                # this is just a fallback in case reading dependencies from setup.py fails
+                self.pip_cmd,
+                "install",
+                "-q",
+                "pyhocon",
+                "prophecy-libs",
+                "pytest",
+                "pytest-html",
+            ]
+        )
+
         return Process.process_sequential(
             [
                 # Extract the install_requires and extra_requires and install them
                 Process(
-                    [self.pip_cmd, "install", "-q"] + python_dependencies,
+                    install_python_dependencies_cmd,
                     path_pipeline_absolute,
                     is_shell=(self.operating_system == "win32"),
                 ),
@@ -638,13 +736,9 @@ class ProphecyBuildTool:
         self.fabric = os.environ.get("FABRIC_NAME")
 
         if self.fabric is None:
-            self._error(
-                "[i]FABRIC_NAME[/i] environment variable is required to "
-                "run your Unit tests"
-            )
-            return False
-        else:
-            return True
+            self.fabric = "default"
+            os.environ["FABRIC_NAME"] = self.fabric
+        return True
 
     def _parse_project(self):
         self.pipelines: Dict = {}
@@ -675,6 +769,7 @@ class ProphecyBuildTool:
         self.jobs_count = len(self.jobs)
 
         jobs_str = ", ".join(map(lambda job: job["name"], self.jobs.values()))
+        print(f"Project name: {self.project['name']}")
         print("Found %s jobs: %s" % (self.jobs_count, jobs_str))
 
         pipelines_str = ", ".join(
@@ -717,13 +812,19 @@ class ProphecyBuildTool:
 
     def _get_spark_parameter_files(self, tasks_list, file_extension):
         result = []
+        package_field = (
+            "spark_jar_task"
+            if self.project_language == "scala"
+            else "python_wheel_task"
+        )
         for task in tasks_list:
-            params = [
-                file.replace("/dbfs/", "dbfs:/")
-                for file in list(task["spark_jar_task"]["parameters"])
-                if file.endswith(file_extension)
-            ]
-            result += params
+            if package_field in task and "parameters" in task[package_field]:
+                params = [
+                    file.replace("/dbfs/", "dbfs:/")
+                    for file in list(task[package_field]["parameters"])
+                    if file.endswith(file_extension)
+                ]
+                result += params
         return result
 
     def _construct_local_config_to_dbfs_config_path(self, json_configs):
@@ -741,15 +842,19 @@ class ProphecyBuildTool:
         return local_config_to_dbfs_config_path_map
 
     @classmethod
-    def _verify_databricks_configs(cls):
+    def _verify_databricks_configs(cls, exit_on_failure=True):
         host = os.environ.get("DATABRICKS_HOST")
         token = os.environ.get("DATABRICKS_TOKEN")
 
         if host is None or token is None:
-            cls._error(
-                "[i]DATABRICKS_HOST[/i] & [i]DATABRICKS_TOKEN[/i] environment variables are required to "
-                "deploy your Databricks Workflows"
-            )
+            if exit_on_failure:
+                cls._error(
+                    "[i]DATABRICKS_HOST[/i] & [i]DATABRICKS_TOKEN[/i] environment variables are required to "
+                    "deploy your Databricks Workflows"
+                )
+            else:
+                return False
+        return True
 
     def _verify_project(self):
         if not os.path.isfile(self.path_project):
